/*****************************************************************************/
/*                      WiNRADiO G65DDC EXCALIBUR SIGMA                      */
/*                                SDK DEMO                                   */
/*                         Simple numeric edit box                           */
/*                        (c) Copyright RADIXON 2020                         */
/*****************************************************************************/

#include "u_numericedit.h"

DEFINE_EVENT_TYPE(wxEVT_COMMAND_NUMERIC_EDIT_CHANGED)

BEGIN_EVENT_TABLE(TNumericEdit, wxTextCtrl)
    EVT_KEY_DOWN(TNumericEdit::OnKeyDown)
    EVT_SET_FOCUS(TNumericEdit::OnSetFocus)
    EVT_KILL_FOCUS(TNumericEdit::OnKillFocus)
    EVT_IDLE(TNumericEdit::OnIdle)
END_EVENT_TABLE()

TNumericEdit::TNumericEdit(wxWindow *Parent,wxWindowID Id,int64_t Value,int64_t MinValue,int64_t MaxValue,const wxPoint &Position,const wxSize &Size):wxTextCtrl(Parent,Id,wxEmptyString,Position,Size,wxTE_RIGHT|wxTE_PROCESS_ENTER)
{
    m_MinValue=MinValue;
    m_MaxValue=MaxValue;
    m_Value=Value;
    m_Editing=false;

    m_SelectAll=false;
}

TNumericEdit::~TNumericEdit(void)
{
}

void TNumericEdit::Set(int64_t Value)
{
    if(Value<m_MinValue)
    {
        Value=m_MinValue;
    }
    else if(Value>m_MaxValue)
    {
        Value=m_MaxValue;
    }

    if(m_Value!=Value)
    {
        m_Value=Value;

        UpdateText();

        FireChangeEvent();
    }
}

void TNumericEdit::SetMinMax(int64_t Min,int64_t Max)
{
    if(m_MinValue!=Min || m_MaxValue!=Max)
    {
        m_MinValue=Min;
        m_MaxValue=Max;

        if(m_Value<Min)
        {
            m_Value=Min;
            UpdateText();
            FireChangeEvent();
        }
        else if(m_Value>Max)
        {
            m_Value=Max;
            UpdateText();
            FireChangeEvent();
        }
    }
}

void TNumericEdit::UpdateText(void)
{
    SetValue(Format(m_Value));
}

void TNumericEdit::FireChangeEvent(void)
{
 wxNotifyEvent Event(wxEVT_COMMAND_NUMERIC_EDIT_CHANGED,GetId());

	ProcessEvent(Event);
}

void TNumericEdit::OnKeyDown(wxKeyEvent &Event)
{
 int64_t Value;
 wxPanel *Panel;

    switch(Event.GetKeyCode())
    {
        case WXK_RETURN:
        case WXK_NUMPAD_ENTER:
            m_Editing=false;

            if(Parse(GetValue(),&Value))
            {
                Set(Value);
            }
            else
            {
                UpdateText();
            }

            Panel=dynamic_cast<wxPanel*>(GetParent());

            if(Panel)
            {
                Panel->SetFocusIgnoringChildren();
            }
            return;

        case WXK_ESCAPE:
            m_Editing=false;

            UpdateText();

            Panel=dynamic_cast<wxPanel*>(GetParent());

            if(Panel)
            {
                Panel->SetFocusIgnoringChildren();
            }
            return;
    }

    Event.Skip();
}

void TNumericEdit::OnSetFocus(wxFocusEvent &Event)
{
    Event.Skip();

    //select all text when focused
    //SelectAll(); - it has to be done when focus event is processed
    m_SelectAll=true; //only set a flag and the SelectAll will be done when idle
    m_Editing=true;
}

void TNumericEdit::OnKillFocus(wxFocusEvent &Event)
{
 int64_t Value;

    Event.Skip();

    if(m_Editing)
    {
        if(Parse(GetValue(),&Value))
        {
            Set(Value);
        }
        else
        {
            UpdateText();
        }

        m_Editing=false;
    }
}

void TNumericEdit::OnIdle(wxIdleEvent &Event)
{
    if(m_SelectAll)
    {
        m_SelectAll=false;

        if(FindFocus()==this)
        {
            SelectAll();
        }
    }

    Event.Skip();
}
