/*****************************************************************************/
/*                      WiNRADiO G65DDC EXCALIBUR SIGMA                      */
/*                       Coherent mode SDK DEMO - ETC                        */
/*                             Main GUI window                               */
/*                        (c) Copyright RADIXON 2023                         */
/*****************************************************************************/

#ifndef __U_MAINWINDOW_H__
#define __U_MAINWINDOW_H__

#include "u_wx.h"
#include "u_defs.h"
#include "u_rx_client.h"
#include <vector>
#include "u_config_window.h"
#include "u_ddc_info.h"
#include "u_spectrum.h"
#include "u_frequencyedit.h"
#include "u_floatedit.h"
#include <atomic>
#include "u_phase.h"

#define UPDATE_INTERVAL 50

#define MAX_FFT_SIZE 8192

class main_window:public wxFrame,public rx_client_callback
{
public:
    void connect_disconnect(void);

    bool is_connected(void) const
    {
        return m_connected;
    }

    void update_rx_button(uint32_t index);

    const std::vector<rx_client*>& get_clients(void) const
    {
        return m_clients;
    }

    virtual void rx_client_callback_ddc1_samples(rx_client *client,uint64_t index,float *samples,uint32_t count,uint32_t bits_per_sample);

    main_window(const std::vector<rx_client*> &clients);
    ~main_window(void);
private:
    std::vector<rx_client*>     m_clients;

    bool                        m_connected;
    bool                        m_power;

    uint32_t                    m_delay;

    wxToggleButton              *m_bpower;
    wxToggleButton              *m_b1pps;
    wxToggleButton              *m_bddc1;
    wxComboBox                  *m_cbddc1_bw;
    TFrequencyEdit              *m_eddc1_frequency;
    wxStaticText                *m_lddc1_rbw;
    TSpectrum                   *m_ddc1_spectrum;
    wxStaticText                *m_latten;
    wxComboBox                  *m_cbatten;
    wxToggleButton              *m_bpreamp;

    config_window               *m_config_window;

    wxTimer                     m_timer;

    std::atomic<uint32_t>       m_active_rx_index;

    uint32_t                    m_ddc1_index;
    struct ddc_info             m_ddc1_info;
    uint32_t                    m_ddc1_sample_sets_per_buffer;
    uint32_t                    m_ddc1_sample_sets_to_send;

    int32_t                     m_ddc1_frequency;

    std::map<uint64_t,uint32_t> m_ddc1_sample_set_index;

    wxCriticalSection           m_ddc1_sample_set_index_cs;

    std::vector<wxToggleButton*>    m_rx_buttons;

    std::vector<float*>         m_ddc1_buffers;

    float                       m_fft_coeffs[MAX_FFT_SIZE];
    uint32_t                    m_fft_size;
    float                       m_fft_buffer[MAX_FFT_SIZE*2];

    wxPanel                     *m_panel;

    std::vector<wxSizer*>       m_phase_sizers;
    std::vector<TPhase**>       m_phases;

    std::vector<wxStaticText*>  m_lphase_shift;
    std::vector<TFloatEdit*>    m_ephase_shift;

    std::vector<wxStaticText*>  m_lstate;
    std::vector<wxStaticText*>  m_lstate_value;

    std::vector<wxStaticText*>  m_l1pps_event_counter;
    std::vector<wxStaticText*>  m_l1pps_event_counter_value;

    std::vector<wxStaticText*>  m_l1pps_adc_period_counter;
    std::vector<wxStaticText*>  m_l1pps_adc_period_counter_value;

    std::vector<wxStaticText*>  m_l1pps_ddc1_sample_counter;
    std::vector<wxStaticText*>  m_l1pps_ddc1_sample_counter_value;

    std::vector<wxStaticText*>  m_l1pps_ddc1_adc_counter;
    std::vector<wxStaticText*>  m_l1pps_ddc1_adc_counter_value;

    void free_buffers(void);

    void connect(void);
    void disconnect(void);
    void connection_state_changed(void);

    void load_settings(void);
    void save_settings(void);


    void on_click_power(wxCommandEvent &event);
    void on_click_1pps(wxCommandEvent &event);
    void on_click_ddc1(wxCommandEvent &event);
    void on_timer(wxTimerEvent& event);
    void on_ddc1_bw_chaned(wxCommandEvent &event);
    void on_ddc1_frequency_changed(wxNotifyEvent &event);
    void on_atten_chaned(wxCommandEvent &event);
    void on_click_preamp(wxCommandEvent &event);
    void on_click_rx_button(wxCommandEvent &event);
    void on_phase_shift_changed(wxNotifyEvent &event);

    void update_state(void);
    void update_ddc1_bw(void);
    void update_device_states(void);

    void set_atten(void);
    void set_preamp(void);
    void set_1pps(void);
    void trigger(void);

    static void get_phase(float *buffer1,float *buffer2,uint32_t count,float *min,float *max);

    DECLARE_EVENT_TABLE()
};

#endif

