/*****************************************************************************/
/*                      WiNRADiO G65DDC EXCALIBUR SIGMA                      */
/*                       Coherent mode SDK DEMO - ETC                        */
/*                         Simple numeric edit box                           */
/*                        (c) Copyright RADIXON 2023                         */
/*****************************************************************************/

#ifndef __U_NUMERICEDIT_H__
#define __U_NUMERICEDIT_H__

#include "u_wx.h"
#include <stdint.h>

BEGIN_DECLARE_EVENT_TYPES()
DECLARE_EVENT_TYPE(wxEVT_COMMAND_NUMERIC_EDIT_CHANGED,wxID_ANY)
END_DECLARE_EVENT_TYPES()

#define EVT_NUMERICEDIT(id, fn)                                             \
	DECLARE_EVENT_TABLE_ENTRY(wxEVT_COMMAND_NUMERIC_EDIT_CHANGED, id, -1,   \
	(wxObjectEventFunction)(wxEventFunction)                                \
	(wxNotifyEventFunction) & fn, (wxObject*) NULL ),

class TNumericEdit:public wxTextCtrl
{
public:
    void Set(int64_t Value);

    inline int64_t Get(void) const
    {
        return m_Value;
    }

    void SetMinMax(int64_t Min,int64_t Max);

    inline int64_t GetMin(void) const
    {
        return m_MinValue;
    }

    inline int64_t GetMax(void) const
    {
        return m_MaxValue;
    }

    TNumericEdit(wxWindow *Parent,wxWindowID Id,int64_t Value,int64_t MinValue,int64_t MaxValue,const wxPoint &Position=wxDefaultPosition,const wxSize &Size=wxDefaultSize);
    virtual ~TNumericEdit(void);
protected:
    virtual bool Parse(const wxString &Str,int64_t *Value)=0;
    virtual wxString Format(int64_t Value)=0;

    void UpdateText(void);
private:
    int64_t             m_MinValue;
    int64_t             m_MaxValue;
    int64_t             m_Value;
    bool                m_Editing;

    bool                m_SelectAll;

    void OnKeyDown(wxKeyEvent &Event);
    void OnSetFocus(wxFocusEvent &Event);
    void OnKillFocus(wxFocusEvent &Event);
    void OnIdle(wxIdleEvent &Event);

    void FireChangeEvent(void);

    DECLARE_EVENT_TABLE()
};

#endif //__U_NUMERICEDIT_H__
