/*****************************************************************************/
/*                      WiNRADiO G65DDC EXCALIBUR SIGMA                      */
/*                       Coherent mode SDK DEMO - ETC                        */
/*                        (c) Copyright RADIXON 2023                         */
/*****************************************************************************/

#ifndef __U_PACKET_U__
#define __U_PACKET_U__

#include <stdint.h>
#include <stddef.h>
#include "g65ddcapi.h"

#pragma pack(push,1)


#define UDP_PACKET_SIZE(__body__)

#define MAX_PACKET_BODY_SIZE        (320*1024)
#define MAX_PACKET_SIZE             (sizeof(packet_size_t)+MAX_PACKET_BODY_SIZE)

#define MAX_DDC1_SAMPLE_SET_COUNT   32768

#define EMPTY_PACKET_SIZE           (sizeof(packet_code_t)+sizeof(packet_seq_t))
#define PACKET_SIZE(__body__)       (EMPTY_PACKET_SIZE+sizeof(((union tcp_packet*)0)->__body__))
#define TOTAL_PACKET_SIZE(__body__) (sizeof(packet_size_t)+PACKET_SIZE(__body__))



typedef uint32_t    packet_size_t;
typedef uint32_t    packet_code_t;
typedef uint32_t    packet_seq_t;

enum
{
    PACKET_CODE_C_SET_EXTENDED_CONFIGURATION_PARAMS =   0x00000100,
    PACKET_CODE_C_SET_POWER,
    PACKET_CODE_C_SET_ATTEN,
    PACKET_CODE_C_SET_PREAMP,
    PACKET_CODE_C_SET_DDC1_FREQUENCY,
    PACKET_CODE_C_SET_DDC1_PHASE_SHIFT,
    PACKET_CODE_C_START_DDC1,
    PACKET_CODE_C_STOP_DDC1,
    PACKET_CODE_C_TRIGGER,
    PACKET_CODE_C_TRIGGER_TEST,
    PACKET_CODE_C_SET_1PPS,

    PACKET_CODE_S_ACK=0x80000000,
    PACKET_CODE_S_DEVICE_INFO,
    PACKET_CODE_S_DDC1_SAMPLES,
    PACKET_CODE_S_STATE
};

union tcp_packet
{
    struct
    {
        struct
        {
            packet_size_t       size;
            packet_code_t       code;
            packet_seq_t        seq;
        };

        union
        {
            struct
            {
                uint32_t        index;
                uint32_t        device_count;
                int32_t         delay;
            } c_extended_configuration_params;

            struct
            {
                uint8_t         power;
            } c_power;

            struct
            {
                int32_t         frequency;
            } c_ddc1_frequency;

            struct
            {
                double          phase_shift;
            } c_ddc1_phase_shift;

            struct
            {
                uint32_t        atten;
            } c_atten;

            struct
            {
                uint8_t         preamp;
            } c_preamp;

            struct
            {
                uint32_t        type_index;
                uint32_t        sample_sets_per_buffer;
                uint32_t        sample_sets_to_send;
            } c_start_ddc1;

            struct
            {
                uint8_t         enabled;
            } c_1pps;

            struct
            {
                packet_code_t   code;
                packet_seq_t    seq;
                uint32_t        error;
            } s_ack;

            struct
            {
                char            serial_number[9];
            } s_device_info;

            struct
            {
                uint64_t        index;
                uint32_t        count;
                uint32_t        bits_per_sample;
                uint8_t         data[1];
            } s_ddc1_samples;

            struct
            {
                uint32_t        event_counter;
                uint32_t        adc_period_counter;
            } s_1pps;

            struct
            {
                double          sample_counter;
                uint64_t        adc_counter;
            } s_1pps_ddc1;

            struct
            {
                uint32_t        event_counter;
                uint32_t        adc_period_counter;
                double          sample_counter;
                uint64_t        adc_counter;
                G65DDC_DEVICE_STATE device;
            } s_state;
        };
    };

    uint8_t u8[1];
    char c[1];
};

#pragma pack(pop)


#endif //__U_PACKET_U__
