#ifndef __G65DDCAPI_H__
#define __G65DDCAPI_H__

#include <stdint.h>

#pragma pack(push,1)

#define G65DDC_CLASS_ID_DEVICE_ENUMERATOR               0x0000F001
#define G65DDC_CLASS_ID_DEVICE                          0x0000F002
#define G65DDC_CLASS_ID_NETWORK_ENUMERATOR              0x0000F003
#define G65DDC_CLASS_ID_COH_DEVICE_SET_ENUMERATOR       0x0000F804
#define G65DDC_CLASS_ID_COH_DEVICE_SET                  0x0000F805

typedef int (*G65DDC_CREATE_INSTANCE)(uint32_t ClassId,void **Interface);

#define G65DDC_OPEN_DEMO                                ((char*)0)
#define G65DDC_OPEN_FIRST                               ((char*)-1)

#define COH_G65DDC_OPEN_DEMO                            ((G65DDC_DEVICE_INFO*)0)
#define COH_G65DDC_OPEN_FIRST                           ((G65DDC_DEVICE_INFO*)-1)

#define COH_G65DDC_CONFIGURATION_NORMAL                 0
#define COH_G65DDC_CONFIGURATION_EXTENDED               1

#define G65DDC_INTERFACE_TYPE_PCIE                      0x00
#define G65DDC_INTERFACE_TYPE_USB2                      0x01
#define G65DDC_INTERFACE_TYPE_USB3                      0x02
#define G65DDC_INTERFACE_TYPE_LAN                       0x03
#define G65DDC_INTERFACE_TYPE_DEMO                      0xFF

#define G65DDC_FRONT_PANEL_LED_MODE_OFF                 0
#define G65DDC_FRONT_PANEL_LED_MODE_ON                  1
#define G65DDC_FRONT_PANEL_LED_MODE_DIAG                2

#define G65DDC_RANGE_1                                  0
#define G65DDC_RANGE_2                                  1
#define G65DDC_RANGE_COUNT                              2

#define G65DDC_DEVICE_STATE_HIGH_TEMPERATURE            0x00000001
//Coherent mode
#define G65DDC_DEVICE_STATE_NO_1PPS                     0x00010000
#define G65DDC_DEVICE_STATE_NOT_ARMED                   0x00020000
#define G65DDC_DEVICE_STATE_TRIGGER_NOT_SENT            0x00040000
#define G65DDC_DEVICE_STATE_NOT_TRIGGERED               0x00080000

#define G65DDC_TEMPERATURE_NA                           (-0x4000)

#define G65DDC_MODE_AM                                  0
#define G65DDC_MODE_AMS                                 1
#define G65DDC_MODE_LSB                                 2
#define G65DDC_MODE_USB                                 3
#define G65DDC_MODE_DSB                                 4
#define G65DDC_MODE_ISB                                 5
#define G65DDC_MODE_CW                                  6
#define G65DDC_MODE_FM                                  7
#define G65DDC_MODE_COUNT                               (G65DDC_MODE_FM+1)

#define G65DDC_SIDE_BAND_LOWER                          1
#define G65DDC_SIDE_BAND_UPPER                          2
#define G65DDC_SIDE_BAND_BOTH                           3

#define G65DDC_AUDIO_STREAM_CALLBACK_STAGE_0            0
#define G65DDC_AUDIO_STREAM_CALLBACK_STAGE_1            1
#define G65DDC_AUDIO_STREAM_CALLBACK_STAGE_2            2

#define G65DDC_DEMODULATOR_PARAM_AMS_SIDE_BAND          0x00000001
#define G65DDC_DEMODULATOR_PARAM_AMS_CAPTURE_RANGE      0x00000002
#define G65DDC_DEMODULATOR_PARAM_CW_FREQUENCY           0x00000003
#define G65DDC_DEMODULATOR_PARAM_DSB_SIDE_BAND          0x00000004
#define G65DDC_DEMODULATOR_PARAM_ISB_SIDE_BAND          0x00000005

#define G65DDC_DEMODULATOR_STATE_AMS_LOCK               0x00000001
#define G65DDC_DEMODULATOR_STATE_AMS_FREQUENCY          0x00000002
#define G65DDC_DEMODULATOR_STATE_AM_DEPTH               0x00000003
#define G65DDC_DEMODULATOR_STATE_TUNE_ERROR             0x00000004
#define G65DDC_DEMODULATOR_STATE_FM_DEVIATION           0x00000005
#define G65DDC_DEMODULATOR_STATE_DSB_LOCK               0x00000006
#define G65DDC_DEMODULATOR_STATE_DSB_FREQUENCY          0x00000007

#define G65DDC_ADC_SAMPLES_PER_SNAPSHOT_64K             0x10000
#define G65DDC_ADC_SAMPLES_PER_SNAPSHOT_128K            0x20000
#define G65DDC_ADC_SAMPLES_PER_SNAPSHOT_256K            0x40000

#define G65DDC_TEMPERATURE_UNKNOWN                      -16384

#define G65DDC_FLAGS_EXTERNAL_REFERENCE_IN              0x00000001
#define G65DDC_FLAGS_AUDIO_OUTPUT                       0x00000002
#define G65DDC_FLAGS_COHERENT                           0x00000004
#define G65DDC_FLAGS_EXTERNAL_REFERENCE_OUT             0x00000008
#define G65DDC_FLAGS_1PPS                               0x00000010

typedef struct
{
    char            DevicePath[256];
    uint8_t         InterfaceType;
    char            SerialNumber[9];
    uint16_t        HWVersion;
    uint16_t        FWVersion[3];
    uint8_t         EEPROMVersion;

    struct
    {
        uint32_t    MinFrequency;
        uint32_t    MaxFrequency;
    }               Ranges[G65DDC_RANGE_COUNT];

    uint32_t    MaxDDC1ChannelCount;
    uint32_t    MaxDDC2ChannelCount;
    uint32_t    MaxDDC1TypeCount;
    uint32_t    MaxDDC2TypeCount;
    uint32_t    MaxDDC2ChannelsPerDDC1Channel;

    uint32_t    Flags;

    uint8_t     MACAddress[6];
} G65DDC_DEVICE_INFO;

typedef struct
{
    uint32_t    Flags;
    int32_t     Temperatures[3];
    uint32_t    FanRPM;
    uint64_t    DataTransferred;
    uint64_t    DataLost;
} G65DDC_DEVICE_STATE;

typedef struct
{
    uint32_t    SampleRate;
    uint32_t    Bandwidth;
    uint32_t    BitsPerSample;
} G65DDC_DDC_INFO;

typedef struct
{
    uint32_t    Tune;   //Hz
    uint32_t    Lock;   //Hz
} G65DDC_AMS_CAPTURE_RANGE;

typedef struct
{
    uint16_t    Size;
    uint32_t    IpAddress;  //network-byte-order
    uint32_t    Mask;       //network-byte-order
} G65DDC_NETWORK_PARAMS;

#ifdef __cplusplus

#define G65DDC_DECLARE_INTERFACE(__name__) struct __name__
#define G65DDC_DECLARE_INTERFACE_2(__name__,__base__) struct __name__ : public __base__
#define G65DDC_THIS
#define G65DDC_METHOD(__type__,__method__,__params__) virtual __type__ __method__ __params__=0
#define G65DDC_METHOD_NO_PARAMS(__type__,__method__) virtual __type__ __method__(void)=0
#else //__cplusplus
#define G65DDC_THIS struct G65DDC_INTERFACE *This,
#define G65DDC_DECLARE_INTERFACE(__name__) typedef struct __name__ { const struct __name__##Vtbl *lpVtbl;} __name__;\
    typedef const struct __name__##Vtbl __name__##Vtbl;struct __name__##Vtbl
#define G65DDC_DECLARE_INTERFACE_2(__name__,__base__) G65DDC_DECLARE_INTERFACE(__name__)
#define G65DDC_METHOD(__type__,__method__,__params__) __type__ (*__method__)__params__
#define G65DDC_METHOD_NO_PARAMS(__type__,__method__) __type__ (*__method__)(struct G65DDC_INTERFACE *This)
#endif //__cplusplus

#undef G65DDC_INTERFACE
#define G65DDC_INTERFACE IG65DDCDeviceEnumerator

G65DDC_DECLARE_INTERFACE(IG65DDCDeviceEnumerator)
{
    G65DDC_METHOD_NO_PARAMS(void,Free);
    G65DDC_METHOD_NO_PARAMS(int,Enumerate);
    G65DDC_METHOD_NO_PARAMS(uint32_t,GetCount);
    G65DDC_METHOD(int,GetDeviceInfo,(G65DDC_THIS uint32_t DeviceIndex,G65DDC_DEVICE_INFO *DeviceInfo));
};


#undef G65DDC_INTERFACE
#define G65DDC_INTERFACE IG65DDCNetworkEnumeratorCallback

G65DDC_DECLARE_INTERFACE(IG65DDCNetworkEnumeratorCallback)
{
    G65DDC_METHOD(void,G65DDC_DeviceFound,(G65DDC_THIS uint32_t IpAddress,uint16_t Port,const char *SerialNumber,uint32_t Flags));
};

#undef G65DDC_INTERFACE
#define G65DDC_INTERFACE IG65DDCNetworkEnumerator

G65DDC_DECLARE_INTERFACE(IG65DDCNetworkEnumerator)
{
    G65DDC_METHOD_NO_PARAMS(void,Free);
    G65DDC_METHOD(int,Start,(G65DDC_THIS IG65DDCNetworkEnumeratorCallback *Callback));
    G65DDC_METHOD_NO_PARAMS(void,Stop);
};

struct IG65DDCDevice;

#undef G65DDC_INTERFACE
#define G65DDC_INTERFACE IG65DDCDeviceCallback

G65DDC_DECLARE_INTERFACE(IG65DDCDeviceCallback)
{
    G65DDC_METHOD(void,G65DDC_ADCSnapshotCallback,(G65DDC_THIS struct IG65DDCDevice *Device,const short *Buffer,uint32_t Count,uint32_t CenterFrequency,uint16_t ADCLevel));
    G65DDC_METHOD(void,G65DDC_DDC1StreamCallback,(G65DDC_THIS struct IG65DDCDevice *Device,uint32_t DDC1ChannelId,const void *Buffer,uint32_t Count,uint32_t BitsPerSample));
    G65DDC_METHOD(int,G65DDC_DDC1PlaybackStreamCallback,(G65DDC_THIS struct IG65DDCDevice *Device,uint32_t DDC1ChannelId,void *Buffer,uint32_t Count,uint32_t BitsPerSample) );
    G65DDC_METHOD(void,G65DDC_DDC2StreamCallback,(G65DDC_THIS struct IG65DDCDevice *Device,uint32_t DDC2ChannelId,const float *Buffer,uint32_t Count));
    G65DDC_METHOD(void,G65DDC_DDC2PreprocessedStreamCallback,(G65DDC_THIS struct IG65DDCDevice *Device,uint32_t DDC2ChannelId,const float *Buffer,uint32_t Count,float Peak,float RMS));
    G65DDC_METHOD(void,G65DDC_AudioStreamCallback,(G65DDC_THIS struct IG65DDCDevice *Device,uint32_t DDC2ChannelId,uint32_t Stage,const float *Buffer,uint32_t Count));
    G65DDC_METHOD(int,G65DDC_AudioPlaybackStreamCallback,(G65DDC_THIS struct IG65DDCDevice *Device,uint32_t DDC2ChannelId,float *Buffer,uint32_t Count));
};

#define G65DDC_DEVICE_INTERFACE_VERSION         0x00000001

#undef G65DDC_INTERFACE
#define G65DDC_INTERFACE IG65DDCDevice

G65DDC_DECLARE_INTERFACE(IG65DDCDevice)
{
    G65DDC_METHOD_NO_PARAMS(int,Free);

    G65DDC_METHOD(void,SetCallback,(G65DDC_THIS IG65DDCDeviceCallback *Callback));
    G65DDC_METHOD_NO_PARAMS(IG65DDCDeviceCallback*,GetCallback);

    G65DDC_METHOD(int,Open,(G65DDC_THIS const char *Path));
    G65DDC_METHOD_NO_PARAMS(int,Close);

    G65DDC_METHOD_NO_PARAMS(int,IsOpen);

    G65DDC_METHOD(int,GetDeviceInfo,(G65DDC_THIS G65DDC_DEVICE_INFO *DeviceInfo));

    G65DDC_METHOD(int,IsConnected,(G65DDC_THIS int *Connected));

    G65DDC_METHOD(int,SetLED,(G65DDC_THIS uint32_t LEDMode));
    G65DDC_METHOD(int,GetLED,(G65DDC_THIS uint32_t *LEDMode));

    G65DDC_METHOD(int,SetPower,(G65DDC_THIS int Power));
    G65DDC_METHOD(int,GetPower,(G65DDC_THIS int *Power));

    G65DDC_METHOD(int,GetDeviceState,(G65DDC_THIS G65DDC_DEVICE_STATE *State));

    G65DDC_METHOD(int,SetExternalReference,(G65DDC_THIS int Enabled));
    G65DDC_METHOD(int,GetExternalReference,(G65DDC_THIS int *Enabled));

    G65DDC_METHOD(int,SetBuiltInTest,(G65DDC_THIS int Enabled));
    G65DDC_METHOD(int,GetBuiltInTest,(G65DDC_THIS int *Enabled));

    G65DDC_METHOD(int,SetAttenuator,(G65DDC_THIS uint32_t Attenuator));
    G65DDC_METHOD(int,GetAttenuator,(G65DDC_THIS uint32_t *Attenuator));

    G65DDC_METHOD(int,SetPreselectors,(G65DDC_THIS uint32_t Low,uint32_t High));
    G65DDC_METHOD(int,GetPreselectors,(G65DDC_THIS uint32_t *Low,uint32_t *High));

    G65DDC_METHOD(int,SetPreamplifier,(G65DDC_THIS int Preamp));
    G65DDC_METHOD(int,GetPreamplifier,(G65DDC_THIS int *Preamp));

    G65DDC_METHOD(int,SetInverted,(G65DDC_THIS int Inverted));
    G65DDC_METHOD(int,GetInverted,(G65DDC_THIS int *Inverted));

    G65DDC_METHOD(int,SetRange,(G65DDC_THIS uint32_t Range));
    G65DDC_METHOD(int,GetRange,(G65DDC_THIS uint32_t *Range));

    G65DDC_METHOD(int,SetADCNoiseBlanker,(G65DDC_THIS int Enabled));
    G65DDC_METHOD(int,GetADCNoiseBlanker,(G65DDC_THIS int *Enabled));
    G65DDC_METHOD(int,SetADCNoiseBlankerThreshold,(G65DDC_THIS uint16_t Threshold));
    G65DDC_METHOD(int,GetADCNoiseBlankerThreshold,(G65DDC_THIS uint16_t *Threshold) );

    G65DDC_METHOD(int,StartADCSnapshots,(G65DDC_THIS uint16_t Interval,uint32_t SamplesPerSnapshot));
    G65DDC_METHOD_NO_PARAMS(int,StopADCSnapshots);

    G65DDC_METHOD(int,GetDDCInfo,(G65DDC_THIS uint32_t DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo));

    G65DDC_METHOD(int,CreateDDC1,(G65DDC_THIS uint32_t *DDC1ChannelId));
    G65DDC_METHOD(int,CreateDDC2,(G65DDC_THIS uint32_t DDC1ChannelId,uint32_t *DDC2ChannelId));

    G65DDC_METHOD(int,DeleteDDC,(G65DDC_THIS uint32_t DDCChannelId));

    G65DDC_METHOD(int,GetDDCCount,(G65DDC_THIS uint32_t DDCChannelId,uint32_t *Count));
    G65DDC_METHOD(int,SetDDC,(G65DDC_THIS uint32_t DDCChannelId,uint32_t TypeIndex));
    G65DDC_METHOD(int,GetDDC,(G65DDC_THIS uint32_t DDCChannelId,uint32_t *TypeIndex,G65DDC_DDC_INFO *DDCInfo));
    G65DDC_METHOD(int,StartDDC,(G65DDC_THIS uint32_t DDCChannelId,uint32_t SampleSetsPerBuffer));
    G65DDC_METHOD(int,StopDDC,(G65DDC_THIS uint32_t DDCChannelId));

    G65DDC_METHOD(int,StartDDC1Playback,(G65DDC_THIS uint32_t DDC1ChannelId,uint32_t SampleSetsPerBuffer,uint32_t BitsPerSample));
    G65DDC_METHOD(int,PauseDDC1Playback,(G65DDC_THIS uint32_t DDC1ChannelId));
    G65DDC_METHOD(int,ResumeDDC1Playback,(G65DDC_THIS uint32_t DDC1ChannelId));

    G65DDC_METHOD(int,SetDDCFrequency,(G65DDC_THIS uint32_t DDCChannelId,int32_t Frequency));
    G65DDC_METHOD(int,GetDDCFrequency,(G65DDC_THIS uint32_t DDCChannelId,int32_t *Frequency));

    G65DDC_METHOD(int,SetDDC2NoiseBlanker,(G65DDC_THIS uint32_t DDC2ChannelId,int Enabled));
    G65DDC_METHOD(int,GetDDC2NoiseBlanker,(G65DDC_THIS uint32_t DDC2ChannelId,int *Enabled) );
    G65DDC_METHOD(int,SetDDC2NoiseBlankerThreshold,(G65DDC_THIS uint32_t DDC2ChannelId,double Threshold));
    G65DDC_METHOD(int,GetDDC2NoiseBlankerThreshold,(G65DDC_THIS uint32_t DDC2ChannelId,double *Threshold));
    G65DDC_METHOD(int,GetDDC2NoiseBlankerExcessValue,(G65DDC_THIS uint32_t DDC2ChannelId,double *Value));

    G65DDC_METHOD(int,SetNotchFilter,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int Enabled));
    G65DDC_METHOD(int,GetNotchFilter,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int *Enabled));
    G65DDC_METHOD(int,SetNotchFilterFrequency,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int32_t Frequency));
    G65DDC_METHOD(int,GetNotchFilterFrequency,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int32_t *Frequency));
    G65DDC_METHOD(int,SetNotchFilterBandwidth,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t Bandwidth));
    G65DDC_METHOD(int,GetNotchFilterBandwidth,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t *Bandwidth));
    G65DDC_METHOD(int,SetNotchFilterLength,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t Length));
    G65DDC_METHOD(int,GetNotchFilterLength,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t *Length));

    G65DDC_METHOD(int,SetAGC,(G65DDC_THIS uint32_t DDC2ChannelId,int Enabled));
    G65DDC_METHOD(int,GetAGC,(G65DDC_THIS uint32_t DDC2ChannelId,int *Enabled));
    G65DDC_METHOD(int,SetAGCParams,(G65DDC_THIS uint32_t DDC2ChannelId,double AttackTime,double DecayTime,double ReferenceLevel));
    G65DDC_METHOD(int,GetAGCParams,(G65DDC_THIS uint32_t DDC2ChannelId,double *AttackTime,double *DecayTime,double *ReferenceLevel));
    G65DDC_METHOD(int,SetMaxAGCGain,(G65DDC_THIS uint32_t DDC2ChannelId,double MaxGain));
    G65DDC_METHOD(int,GetMaxAGCGain,(G65DDC_THIS uint32_t DDC2ChannelId,double *MaxGain));
    G65DDC_METHOD(int,SetGain,(G65DDC_THIS uint32_t DDC2ChannelId,double Gain));
    G65DDC_METHOD(int,GetGain,(G65DDC_THIS uint32_t DDC2ChannelID,double *Gain));
    G65DDC_METHOD(int,GetCurrentGain,(G65DDC_THIS uint32_t DDC2ChannelId,double *Gain));

    G65DDC_METHOD(int,GetSignalLevel,(G65DDC_THIS uint32_t DDC2ChannelId,float *Peak,float *RMS) );

    G65DDC_METHOD(int,SetDemodulatorMode,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t Mode) );
    G65DDC_METHOD(int,GetDemodulatorMode,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t *Mode) );
    G65DDC_METHOD(int,SetDemodulatorFrequency,(G65DDC_THIS uint32_t DDC2ChannelId,int32_t Frequency) );
    G65DDC_METHOD(int,GetDemodulatorFrequency,(G65DDC_THIS uint32_t DDC2ChannelId,int32_t *Frequency) );
    G65DDC_METHOD(int,SetDemodulatorFilterBandwidth,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t Bandwidth) );
    G65DDC_METHOD(int,GetDemodulatorFilterBandwidth,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t *Bandwidth) );
    G65DDC_METHOD(int,SetDemodulatorFilterShift,(G65DDC_THIS uint32_t DDC2ChannelId,int32_t Shift) );
    G65DDC_METHOD(int,GetDemodulatorFilterShift,(G65DDC_THIS uint32_t DDC2ChannelId,int32_t *Shift) );
    G65DDC_METHOD(int,SetDemodulatorFilterLength,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t Length) );
    G65DDC_METHOD(int,GetDemodulatorFilterLength,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t *Length) );

    G65DDC_METHOD(int,SetDemodulatorParam,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t Code,const void *Buffer,uint32_t BufferSize) );
    G65DDC_METHOD(int,GetDemodulatorParam,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t Code,void *Buffer,uint32_t BufferSize) );
    G65DDC_METHOD(int,GetDemodulatorState,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t Code,void *Buffer,uint32_t BufferSize) );

    G65DDC_METHOD(int,StartAudio,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t SampleSetsPerBuffer) );
    G65DDC_METHOD(int,StopAudio,(G65DDC_THIS uint32_t DDC2ChannelId) );

    G65DDC_METHOD(int,StartAudioPlayback,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t SampleSetsPerBuffer) );
    G65DDC_METHOD(int,PauseAudioPlayback,(G65DDC_THIS uint32_t DDC2ChannelId) );
    G65DDC_METHOD(int,ResumeAudioPlayback,(G65DDC_THIS uint32_t DDC2ChannelId) );

    G65DDC_METHOD(int,SetAudioFilter,(G65DDC_THIS uint32_t DDC2ChannelId,int Enabled) );
    G65DDC_METHOD(int,GetAudioFilter,(G65DDC_THIS uint32_t DDC2ChannelId,int *Enabled) );
    G65DDC_METHOD(int,SetAudioFilterParams,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t CutOffLow,uint32_t CutOffHigh,double Deemphasis) );
    G65DDC_METHOD(int,GetAudioFilterParams,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t *CutOffLow,uint32_t *CutOffHigh,double *Deemphasis) );
    G65DDC_METHOD(int,SetAudioFilterLength,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t Length) );
    G65DDC_METHOD(int,GetAudioFilterLength,(G65DDC_THIS uint32_t DDC2ChannelId,uint32_t *Length) );
    G65DDC_METHOD(int,SetAudioGain,(G65DDC_THIS uint32_t DDC2ChannelId,double Gain) );
    G65DDC_METHOD(int,GetAudioGain,(G65DDC_THIS uint32_t DDC2ChannelId,double *Gain) );
    G65DDC_METHOD(int,SetVolume,(G65DDC_THIS uint32_t DDC2ChannelId,uint8_t Volume) );
    G65DDC_METHOD(int,GetVolume,(G65DDC_THIS uint32_t DDC2ChannelId,uint8_t *Volume) );
    G65DDC_METHOD(int,SetMute,(G65DDC_THIS uint32_t DDC2ChannelId,int Mute) );
    G65DDC_METHOD(int,GetMute,(G65DDC_THIS uint32_t DDC2ChannelId,int *Mute) );


    G65DDC_METHOD(int,SetFrequency,(G65DDC_THIS uint32_t DDC2ChannelId,int32_t Frequency) );
    G65DDC_METHOD(int,GetFrequency,(G65DDC_THIS uint32_t DDC2ChannelId,int32_t *Frequency) );

    G65DDC_METHOD(int,GetSpectrumCompensation,(G65DDC_THIS int32_t Frequency,uint32_t Bandwidth,float *Buffer,uint32_t Count) );

    G65DDC_METHOD(int,SetNetworkParams,(G65DDC_THIS const G65DDC_NETWORK_PARAMS *Params) );
    G65DDC_METHOD(int,GetNetworkParams,(G65DDC_THIS G65DDC_NETWORK_PARAMS *Params) );

    G65DDC_METHOD(int,GetInterfaceVersion,(G65DDC_THIS uint32_t *Version) );

    G65DDC_METHOD(int,Get1PPSCounters,(G65DDC_THIS uint32_t *EventCounter,uint32_t *ADCPeriodCounter) );
    G65DDC_METHOD(int,Get1PPSDDC1Counters,(G65DDC_THIS uint32_t DDC1ChannelId,double *SampleCounter,uint64_t *ADCPeriodCounter) );

    G65DDC_METHOD(int,SetExternalADCClock,(G65DDC_THIS int Enabled) );
    G65DDC_METHOD(int,GetExternalADCClock,(G65DDC_THIS int *Enabled) );

    G65DDC_METHOD(int,SetDAC,(G65DDC_THIS uint32_t DDC2ChannelId,int Enabled) );
    G65DDC_METHOD(int,GetDAC,(G65DDC_THIS uint32_t DDC2ChannelId,int *Enabled) );
};



#undef G65DDC_INTERFACE
#define G65DDC_INTERFACE ICohG65DDCDeviceSetEnumerator

G65DDC_DECLARE_INTERFACE(ICohG65DDCDeviceSetEnumerator)
{
    G65DDC_METHOD_NO_PARAMS(void,Free);
    G65DDC_METHOD_NO_PARAMS(int,Enumerate);
    G65DDC_METHOD_NO_PARAMS(uint32_t,GetDeviceSetCount);
    G65DDC_METHOD(int,GetDeviceSetInfo,(G65DDC_THIS uint32_t DeviceSetIndex,G65DDC_DEVICE_INFO *DeviceInfos,uint32_t *DeviceInfoCount));
};


#undef G65DDC_INTERFACE
#define G65DDC_INTERFACE ICohG65DDCDeviceSetCallback

struct ICohG65DDCDeviceSet;

G65DDC_DECLARE_INTERFACE(ICohG65DDCDeviceSetCallback)
{
    G65DDC_METHOD(void,CohG65DDC_ADCSnapshotCallback,(G65DDC_THIS struct ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,const short *Buffer,uint32_t Count,uint32_t CenterFrequency,uint16_t ADCLevel));
    G65DDC_METHOD(void,CohG65DDC_DDC1StreamCallback,(struct ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceCount,const void **Buffers,uint32_t Count,uint32_t BitsPerSample));
    G65DDC_METHOD(void,CohG65DDC_DDC2StreamCallback,(struct ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,const float *Buffer,uint32_t Count));
    G65DDC_METHOD(void,CohG65DDC_DDC2PreprocessedStreamCallback,(struct ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,const float *Buffer,uint32_t Count,float SlevelPeak,float SlevelRMS));
    G65DDC_METHOD(void,CohG65DDC_AudioStreamCallback,(struct ICohG65DDCDeviceSet *DeviceSet,uint32_t DeviceIndex,uint32_t Stage,const float *Buffer,uint32_t Count));
};

#define COH_G65DDC_DEVICE_SET_INTERFACE_VERSION     0x00000001

#undef G65DDC_INTERFACE
#define G65DDC_INTERFACE ICohG65DDCDeviceSet

G65DDC_DECLARE_INTERFACE(ICohG65DDCDeviceSet)
{
    G65DDC_METHOD_NO_PARAMS(void,Free);

    G65DDC_METHOD(void,SetCallback,(ICohG65DDCDeviceSetCallback *Callback));
    G65DDC_METHOD_NO_PARAMS(ICohG65DDCDeviceSetCallback*,GetCallback);

    G65DDC_METHOD(int,Open,(G65DDC_DEVICE_INFO *DeviceInfos,uint32_t Count));
    G65DDC_METHOD_NO_PARAMS(int,Close);

    G65DDC_METHOD_NO_PARAMS(int,IsOpen);

    G65DDC_METHOD(int,GetDeviceCount,(uint32_t *Count));

    G65DDC_METHOD(int,GetDeviceInfo,(G65DDC_THIS uint32_t DeviceIndex,G65DDC_DEVICE_INFO *DeviceInfo));
    G65DDC_METHOD(int,GetDeviceInfos,(G65DDC_THIS G65DDC_DEVICE_INFO *Infos,uint32_t *InfoCount));

    G65DDC_METHOD(int,IsConnected,(G65DDC_THIS int *Connected));

    G65DDC_METHOD(int,SetLED,(G65DDC_THIS uint32_t DeviceIndex,uint32_t LEDMode));
    G65DDC_METHOD(int,GetLED,(G65DDC_THIS uint32_t DeviceIndex,uint32_t *LEDMode));

    G65DDC_METHOD(int,SetPower,(G65DDC_THIS int Power));
    G65DDC_METHOD(int,GetPower,(G65DDC_THIS int *Power));

    G65DDC_METHOD(int,GetDeviceState,(G65DDC_THIS uint32_t DeviceIndex,G65DDC_DEVICE_STATE *State));

    G65DDC_METHOD(int,SetBuiltInTest,(G65DDC_THIS uint32_t DeviceIndex,int Enabled));
    G65DDC_METHOD(int,GetBuiltInTest,(G65DDC_THIS uint32_t DeviceIndex,int *Enabled));

    G65DDC_METHOD(int,SetAttenuator,(G65DDC_THIS uint32_t Attenuator));
    G65DDC_METHOD(int,GetAttenuator,(G65DDC_THIS uint32_t *Attenuator));

    G65DDC_METHOD(int,SetPreselectors,(G65DDC_THIS uint32_t Low,uint32_t High));
    G65DDC_METHOD(int,GetPreselectors,(G65DDC_THIS uint32_t *Low,uint32_t *High));

    G65DDC_METHOD(int,SetPreamplifier,(G65DDC_THIS int Preamp));
    G65DDC_METHOD(int,GetPreamplifier,(G65DDC_THIS int *Preamp));

    G65DDC_METHOD(int,SetInverted,(G65DDC_THIS int Inverted));
    G65DDC_METHOD(int,GetInverted,(G65DDC_THIS int *Inverted));

    G65DDC_METHOD(int,SetRange,(G65DDC_THIS uint32_t Range));
    G65DDC_METHOD(int,GetRange,(G65DDC_THIS uint32_t *Range));

    G65DDC_METHOD(int,SetADCNoiseBlanker,(G65DDC_THIS int Enabled));
    G65DDC_METHOD(int,GetADCNoiseBlanker,(G65DDC_THIS int *Enabled));
    G65DDC_METHOD(int,SetADCNoiseBlankerThreshold,(G65DDC_THIS uint16_t Threshold));
    G65DDC_METHOD(int,GetADCNoiseBlankerThreshold,(G65DDC_THIS uint16_t *Threshold));

    G65DDC_METHOD(int,StartADCSnapshots,(G65DDC_THIS uint32_t DeviceIndex,uint16_t Interval,uint32_t SamplesPerSnapshot));
    G65DDC_METHOD(int,StopADCSnapshots,(G65DDC_THIS uint32_t DeviceIndex));

    G65DDC_METHOD(int,GetDDCInfo,(G65DDC_THIS uint32_t DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo));

    G65DDC_METHOD(int,GetDDC1Count,(G65DDC_THIS uint32_t *Count));
    G65DDC_METHOD(int,SetDDC1,(G65DDC_THIS uint32_t DDCTypeIndex));
    G65DDC_METHOD(int,GetDDC1,(G65DDC_THIS uint32_t *DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo));
    G65DDC_METHOD(int,StartDDC1,(G65DDC_THIS uint32_t SampleSetsPerBuffer));
    G65DDC_METHOD_NO_PARAMS(int,StopDDC1);
    G65DDC_METHOD(int,SetDDC1Frequency,(G65DDC_THIS int32_t Frequency));
    G65DDC_METHOD(int,GetDDC1Frequency,(G65DDC_THIS int32_t *Frequency));
    G65DDC_METHOD(int,SetDDC1PhaseShift,(G65DDC_THIS uint32_t DeviceIndex,double PhaseShift));
    G65DDC_METHOD(int,GetDDC1PhaseShift,(G65DDC_THIS uint32_t DeviceIndex,double *PhaseShift));

    G65DDC_METHOD(int,GetDDC2Count,(G65DDC_THIS uint32_t *Count));
    G65DDC_METHOD(int,SetDDC2,(G65DDC_THIS uint32_t DeviceIndex,uint32_t DDCTypeIndex));
    G65DDC_METHOD(int,GetDDC2,(G65DDC_THIS uint32_t DeviceIndex,uint32_t *DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo));
    G65DDC_METHOD(int,StartDDC2,(G65DDC_THIS uint32_t DeviceIndex,uint32_t SampleSetsPerBuffer));
    G65DDC_METHOD(int,StopDDC2,(G65DDC_THIS uint32_t DeviceIndex));
    G65DDC_METHOD(int,SetDDC2Frequency,(G65DDC_THIS uint32_t DeviceIndex,int32_t Frequency));
    G65DDC_METHOD(int,GetDDC2Frequency,(G65DDC_THIS uint32_t DeviceIndex,int32_t *Frequency));

    G65DDC_METHOD(int,SetDDC2NoiseBlanker,(G65DDC_THIS uint32_t DeviceIndex,int Enabled));
    G65DDC_METHOD(int,GetDDC2NoiseBlanker,(G65DDC_THIS uint32_t DeviceIndex,int *Enabled));
    G65DDC_METHOD(int,SetDDC2NoiseBlankerThreshold,(G65DDC_THIS uint32_t DeviceIndex,double Threshold));
    G65DDC_METHOD(int,GetDDC2NoiseBlankerThreshold,(G65DDC_THIS uint32_t DeviceIndex,double *Threshold));
    G65DDC_METHOD(int,GetDDC2NoiseBlankerExcessValue,(G65DDC_THIS uint32_t DeviceIndex,double *Value));

    G65DDC_METHOD(int,SetNotchFilter,(G65DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,int Enabled));
    G65DDC_METHOD(int,GetNotchFilter,(G65DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,int *Enabled));
    G65DDC_METHOD(int,SetNotchFilterFrequency,(G65DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,int32_t Frequency));
    G65DDC_METHOD(int,GetNotchFilterFrequency,(G65DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,int32_t *Frequency));
    G65DDC_METHOD(int,SetNotchFilterBandwidth,(G65DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t Bandwidth));
    G65DDC_METHOD(int,GetNotchFilterBandwidth,(G65DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t *Bandwidth));
    G65DDC_METHOD(int,SetNotchFilterLength,(G65DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t Length));
    G65DDC_METHOD(int,GetNotchFilterLength,(G65DDC_THIS uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t *Length));

    G65DDC_METHOD(int,SetAGC,(G65DDC_THIS uint32_t DeviceIndex,int Enabled));
    G65DDC_METHOD(int,GetAGC,(G65DDC_THIS uint32_t DeviceIndex,int *Enabled));
    G65DDC_METHOD(int,SetAGCParams,(G65DDC_THIS uint32_t DeviceIndex,double AttackTime,double DecayTime,double ReferenceLevel));
    G65DDC_METHOD(int,GetAGCParams,(G65DDC_THIS uint32_t DeviceIndex,double *AttackTime,double *DecayTime,double *ReferenceLevel));
    G65DDC_METHOD(int,SetMaxAGCGain,(G65DDC_THIS uint32_t DeviceIndex,double MaxGain));
    G65DDC_METHOD(int,GetMaxAGCGain,(G65DDC_THIS uint32_t DeviceIndex,double *MaxGain));
    G65DDC_METHOD(int,SetGain,(G65DDC_THIS uint32_t DeviceIndex,double Gain));
    G65DDC_METHOD(int,GetGain,(G65DDC_THIS uint32_t DeviceIndex,double *Gain));
    G65DDC_METHOD(int,GetCurrentGain,(G65DDC_THIS uint32_t DeviceIndex,double *Gain));

    G65DDC_METHOD(int,GetSignalLevel,(G65DDC_THIS uint32_t DeviceIndex,float *Peak,float *RMS));

    G65DDC_METHOD(int,SetDemodulatorMode,(G65DDC_THIS uint32_t DeviceIndex,uint32_t Mode));
    G65DDC_METHOD(int,GetDemodulatorMode,(G65DDC_THIS uint32_t DeviceIndex,uint32_t *Mode));
    G65DDC_METHOD(int,SetDemodulatorFrequency,(G65DDC_THIS uint32_t DeviceIndex,int32_t Frequency));
    G65DDC_METHOD(int,GetDemodulatorFrequency,(G65DDC_THIS uint32_t DeviceIndex,int32_t *Frequency));
    G65DDC_METHOD(int,SetDemodulatorFilterBandwidth,(G65DDC_THIS uint32_t DeviceIndex,uint32_t Bandwidth));
    G65DDC_METHOD(int,GetDemodulatorFilterBandwidth,(G65DDC_THIS uint32_t DeviceIndex,uint32_t *Bandwidth));
    G65DDC_METHOD(int,SetDemodulatorFilterShift,(G65DDC_THIS uint32_t DeviceIndex,int32_t Shift));
    G65DDC_METHOD(int,GetDemodulatorFilterShift,(G65DDC_THIS uint32_t DeviceIndex,int32_t *Shift));
    G65DDC_METHOD(int,SetDemodulatorFilterLength,(G65DDC_THIS uint32_t DeviceIndex,uint32_t Length));
    G65DDC_METHOD(int,GetDemodulatorFilterLength,(G65DDC_THIS uint32_t DeviceIndex,uint32_t *Length));

    G65DDC_METHOD(int,SetDemodulatorParam,(G65DDC_THIS uint32_t DeviceIndex,uint32_t Code,const void *Buffer,uint32_t BufferSize));
    G65DDC_METHOD(int,GetDemodulatorParam,(G65DDC_THIS uint32_t DeviceIndex,uint32_t Code,void *Buffer,uint32_t BufferSize));
    G65DDC_METHOD(int,GetDemodulatorState,(G65DDC_THIS uint32_t DeviceIndex,uint32_t Code,void *Buffer,uint32_t BufferSize));

    G65DDC_METHOD(int,StartAudio,(G65DDC_THIS uint32_t DeviceIndex,uint32_t SampleSetsPerBuffer));
    G65DDC_METHOD(int,StopAudio,(G65DDC_THIS uint32_t DeviceIndex));

    G65DDC_METHOD(int,SetAudioFilter,(G65DDC_THIS uint32_t DeviceIndex,int Enabled));
    G65DDC_METHOD(int,GetAudioFilter,(G65DDC_THIS uint32_t DeviceIndex,int *Enabled));
    G65DDC_METHOD(int,SetAudioFilterParams,(G65DDC_THIS uint32_t DeviceIndex,uint32_t CutOffLow,uint32_t CutOffHigh,double Deemphasis));
    G65DDC_METHOD(int,GetAudioFilterParams,(G65DDC_THIS uint32_t DeviceIndex,uint32_t *CutOffLow,uint32_t *CutOffHigh,double *Deemphasis));
    G65DDC_METHOD(int,SetAudioFilterLength,(G65DDC_THIS uint32_t DeviceIndex,uint32_t Length));
    G65DDC_METHOD(int,GetAudioFilterLength,(G65DDC_THIS uint32_t DeviceIndex,uint32_t *Length));
    G65DDC_METHOD(int,SetAudioGain,(G65DDC_THIS uint32_t DeviceIndex,double Gain));
    G65DDC_METHOD(int,GetAudioGain,(G65DDC_THIS uint32_t DeviceIndex,double *Gain));
    G65DDC_METHOD(int,SetVolume,(G65DDC_THIS uint32_t DeviceIndex,uint8_t Volume));
    G65DDC_METHOD(int,GetVolume,(G65DDC_THIS uint32_t DeviceIndex,uint8_t *Volume));
    G65DDC_METHOD(int,SetMute,(G65DDC_THIS uint32_t DeviceIndex,int Mute));
    G65DDC_METHOD(int,GetMute,(G65DDC_THIS uint32_t DeviceIndex,int *Mute));

    G65DDC_METHOD(int,GetSpectrumCompensation,(G65DDC_THIS uint32_t DeviceIndex,int32_t Frequency,uint32_t Bandwidth,float *Buffer,uint32_t Count));

    G65DDC_METHOD(int,GetInterfaceVersion,(G65DDC_THIS uint32_t *Version));

    G65DDC_METHOD(int,Set1PPSEnabled,(G65DDC_THIS int Enabled));
    G65DDC_METHOD(int,Get1PPSEnabled,(G65DDC_THIS int *Enabled));
    G65DDC_METHOD(int,Get1PPSCounters,(G65DDC_THIS uint32_t *EventCounter,uint32_t *ADCPeriodCounter));
    G65DDC_METHOD(int,Get1PPSDDC1Counters,(G65DDC_THIS double *SampleCounter,uint64_t *ADCPeriodCounter));

    G65DDC_METHOD(int,SetTriggerDelay,(G65DDC_THIS uint32_t Delay));
    G65DDC_METHOD(int,GetTriggerDelay,(G65DDC_THIS uint32_t *Delay));

    G65DDC_METHOD(int,SetConfiguration,(G65DDC_THIS uint32_t Configuration));
    G65DDC_METHOD(int,GetConfiguration,(G65DDC_THIS uint32_t *Configuration));
    G65DDC_METHOD(int,SetExtendedConfigurationParams,(G65DDC_THIS uint32_t Index,uint32_t DeviceCount));
    G65DDC_METHOD(int,GetExtendedConfigurationParams,(G65DDC_THIS uint32_t *Index,uint32_t *DeviceCount));
    G65DDC_METHOD_NO_PARAMS(int,Trigger);
    G65DDC_METHOD_NO_PARAMS(int,TriggerTest);

    G65DDC_METHOD(int,SetDAC,(G65DDC_THIS uint32_t DeviceIndex,int Enabled));
    G65DDC_METHOD(int,GetDAC,(G65DDC_THIS uint32_t DeviceIndex,int *Enabled));
};

typedef void (*G65DDC_ADC_SNAPSHOT_CALLBACK)(const short *Buffer,uint32_t Count,uint32_t CenterFrequency,uint16_t ADCLevel,uintptr_t UserData);
typedef void (*G65DDC_DDC1_STREAM_CALLBACK)(uint32_t DDC1ChannelId,const void *Buffer,uint32_t Count,uint32_t BitsPerSample,uintptr_t UserData);
typedef int (*G65DDC_DDC1_PLAYBACK_STREAM_CALLBACK)(uint32_t DDC1ChannelId,void *Buffer,uint32_t Count,uint32_t BitsPerSample,uintptr_t UserData);
typedef void (*G65DDC_DDC2_STREAM_CALLBACK)(uint32_t DDC2ChannelId,const float *Buffer,uint32_t Count,uintptr_t UserData);
typedef void (*G65DDC_DDC2_PREPROCESSED_STREAM_CALLBACK)(uint32_t DDC2ChannelId,const float *Buffer,uint32_t Count,float Peak,float RMS,uintptr_t UserData);
typedef void (*G65DDC_AUDIO_STREAM_CALLBACK)(uint32_t DDC2ChannelId,uint32_t Stage,const float *Buffer,uint32_t Count,uintptr_t UserData);
typedef int (*G65DDC_AUDIO_PLAYBACK_STREAM_CALLBACK)(uint32_t DDC2ChannelId,float *Buffer,uint32_t Count,uintptr_t UserData);
typedef int (*G65DDC_SEARCH_NETWORK_DEVICES_CALLBACK)(uint32_t IpAddress,uint16_t Port,const char *SerialNumber,uint32_t Flags,uintptr_t UserData);

typedef struct
{
    G65DDC_ADC_SNAPSHOT_CALLBACK                ADCSnapshotCallback;
    G65DDC_DDC1_STREAM_CALLBACK                 DDC1StreamCallback;
    G65DDC_DDC1_PLAYBACK_STREAM_CALLBACK        DDC1PlaybackStreamCallback;
    G65DDC_DDC2_STREAM_CALLBACK                 DDC2StreamCallback;
    G65DDC_DDC2_PREPROCESSED_STREAM_CALLBACK    DDC2PreprocessedStreamCallback;
    G65DDC_AUDIO_STREAM_CALLBACK                AudioStreamCallback;
    G65DDC_AUDIO_PLAYBACK_STREAM_CALLBACK       AudioPlaybackStreamCallback;
} G65DDC_CALLBACKS;

#pragma pack(pop)

typedef int (*G65DDC_GET_DEVICE_LIST)(G65DDC_DEVICE_INFO **DeviceList,uint32_t *Count);
typedef void (*G65DDC_FREE_DEVICE_LIST)(G65DDC_DEVICE_INFO *DeviceList);
typedef int (*G65DDC_SEARCH_NETWORK_DEVICES)(G65DDC_SEARCH_NETWORK_DEVICES_CALLBACK Callback,uintptr_t UserData,uint32_t Timeout);
typedef int32_t (*G65DDC_OPEN_DEVICE)(const char *SerialNumber);
typedef int (*G65DDC_CLOSE_DEVICE)(int32_t hDevice);
typedef int (*G65DDC_IS_DEVICE_CONNECTED)(int32_t hDevice,int *Connected);
typedef int (*G65DDC_SET_CALLBACKS)(int32_t hDevice,const G65DDC_CALLBACKS *Callbacks,uintptr_t UserData);
typedef int (*G65DDC_GET_DEVICE_INFO)(int32_t hDevice,G65DDC_DEVICE_INFO *DeviceInfo);
typedef int (*G65DDC_SET_LED)(int32_t hDevice,uint32_t LEDMode);
typedef int (*G65DDC_GET_LED)(int32_t hDevice,uint32_t *LEDMode);
typedef int (*G65DDC_SET_POWER)(int32_t hDevice,int Power);
typedef int (*G65DDC_GET_POWER)(int32_t hDevice,int *Power);
typedef int (*G65DDC_GET_DEVICE_STATE)(int32_t hDevice,G65DDC_DEVICE_STATE *State);
typedef int (*G65DDC_SET_EXTERNAL_REFERENCE)(int32_t hDevice,int Enabled);
typedef int (*G65DDC_GET_EXTERNAL_REFERENCE)(int32_t hDevice,int *Enabled);
typedef int (*G65DDC_SET_BUILT_IN_TEST)(int32_t hDevice,int Enabled);
typedef int (*G65DDC_GET_BUILT_IN_TEST)(int32_t hDevice,int *Enabled);
typedef int (*G65DDC_SET_ATTENUATOR)(int32_t hDevice,uint32_t Attenuator);
typedef int (*G65DDC_GET_ATTENUATOR)(int32_t hDevice,uint32_t *Attenuator);
typedef int (*G65DDC_SET_PRESELECTOS)(int32_t hDevice,uint32_t Low,uint32_t High);
typedef int (*G65DDC_GET_PRESELECTOS)(int32_t hDevice,uint32_t *Low,uint32_t *High);
typedef int (*G65DDC_SET_PREAMPLIFIER)(int32_t hDevice,int Preamp);
typedef int (*G65DDC_GET_PREAMPLIFIER)(int32_t hDevice,int *Preamp);
typedef int (*G65DDC_SET_RANGE)(int32_t hDevice,uint32_t Range);
typedef int (*G65DDC_GET_RANGE)(int32_t hDevice,uint32_t *Range);
typedef int (*G65DDC_SET_ADC_NOISE_BLANKER)(int32_t hDevice,int Enabled);
typedef int (*G65DDC_GET_ADC_NOISE_BLANKER)(int32_t hDevice,int *Enabled);
typedef int (*G65DDC_SET_ADC_NOISE_BLANKER_THRESHOLD)(int32_t hDevice,uint16_t Threshold);
typedef int (*G65DDC_GET_ADC_NOISE_BLANKER_THRESHOLD)(int32_t hDevice,uint16_t *Threshold);
typedef int (*G65DDC_START_ADC_SNAPSHOTS)(int32_t hDevice,uint16_t Interval,uint32_t SamplesPerSnapshot);
typedef int (*G65DDC_STOP_ADC_SNAPSHOTS)(int32_t hDevice);
typedef int (*G65DDC_GET_DDC_INFO)(int32_t hDevice,uint32_t DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo);
typedef int (*G65DDC_CREATE_DDC1)(int32_t hDevice,uint32_t *DDC1ChannelId);
typedef int (*G65DDC_CREATE_DDC2)(int32_t hDevice,uint32_t DDC1ChannelId,uint32_t *DDC2ChannelId);
typedef int (*G65DDC_DELETE_DDC)(int32_t hDevice,uint32_t DDCChannelId);
typedef int (*G65DDC_GET_DDC_COUNT)(int32_t hDevice,uint32_t DDCChannelId,uint32_t *Count);
typedef int (*G65DDC_SET_DDC)(int32_t hDevice,uint32_t DDCChannelId,uint32_t TypeIndex);
typedef int (*G65DDC_GET_DDC)(int32_t hDevice,uint32_t DDCChannelId,uint32_t *TypeIndex,G65DDC_DDC_INFO *DDCInfo);
typedef int (*G65DDC_START_DDC)(int32_t hDevice,uint32_t DDCChannelId,uint32_t SampleSetsPerBuffer);
typedef int (*G65DDC_STOP_DDC)(int32_t hDevice,uint32_t DDCChannelId);
typedef int (*G65DDC_START_DDC1_PLAYBACK)(int32_t hDevice,uint32_t DDC1ChannelId,uint32_t SampleSetsPerBuffer,uint32_t BitsPerSample);
typedef int (*G65DDC_PAUSE_DDC1_PLAYBACK)(int32_t hDevice,uint32_t DDC1ChannelId);
typedef int (*G65DDC_RESUME_DDC1_PLAYBACK)(int32_t hDevice,uint32_t DDC1ChannelId);
typedef int (*G65DDC_SET_DDC_FREQUENCY)(int32_t hDevice,uint32_t DDCChannelId,int32_t Frequency);
typedef int (*G65DDC_GET_DDC_FREQUENCY)(int32_t hDevice,uint32_t DDCChannelId,int32_t *Frequency);
typedef int (*G65DDC_SET_DDC2_NOISE_BLANKER)(int32_t hDevice,uint32_t DDC2ChannelId,int Enabled);
typedef int (*G65DDC_GET_DDC2_NOISE_BLANKER)(int32_t hDevice,uint32_t DDC2ChannelId,int *Enabled);
typedef int (*G65DDC_SET_DDC2_NOISE_BLANKER_THREASHOLD)(int32_t hDevice,uint32_t DDC2ChannelId,double Threshold);
typedef int (*G65DDC_GET_DDC2_NOISE_BLANKER_THREASHOLD)(int32_t hDevice,uint32_t DDC2ChannelId,double *Threshold);
typedef int (*G65DDC_GET_DDC2_NOISE_BLANKER_EXCESS_VALUE)(int32_t hDevice,uint32_t DDC2ChannelId,double *Value);
typedef int (*G65DDC_SET_NOTCH_FILTER)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int Enabled);
typedef int (*G65DDC_GET_NOTCH_FILTER)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int *Enabled);
typedef int (*G65DDC_SET_NOTCH_FILTER_FREQUENCY)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int32_t Frequency);
typedef int (*G65DDC_GET_NOTCH_FILTER_FREQUENCY)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,int32_t *Frequency);
typedef int (*G65DDC_SET_NOTCH_FILTER_BANDWIDTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t Bandwidth);
typedef int (*G65DDC_GET_NOTCH_FILTER_BANDWIDTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t *Bandwidth);
typedef int (*G65DDC_SET_NOTCH_FILTER_LENGTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t Length);
typedef int (*G65DDC_GET_NOTCH_FILTER_LENGTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t NotchFilterIndex,uint32_t *Length);
typedef int (*G65DDC_SET_AGC)(int32_t hDevice,uint32_t DDC2ChannelId,int Enabled);
typedef int (*G65DDC_GET_AGC)(int32_t hDevice,uint32_t DDC2ChannelId,int *Enabled);
typedef int (*G65DDC_SET_AGC_PARAMS)(int32_t hDevice,uint32_t DDC2ChannelId,double AttackTime,double DecayTime,double ReferenceLevel);
typedef int (*G65DDC_GET_AGC_PARAMS)(int32_t hDevice,uint32_t DDC2ChannelId,double *AttackTime,double *DecayTime,double *ReferenceLevel);
typedef int (*G65DDC_SET_MAX_AGC_GAIN)(int32_t hDevice,uint32_t DDC2ChannelId,double MaxGain);
typedef int (*G65DDC_GET_MAX_AGC_GAIN)(int32_t hDevice,uint32_t DDC2ChannelId,double *MaxGain);
typedef int (*G65DDC_SET_GAIN)(int32_t hDevice,uint32_t DDC2ChannelId,double Gain);
typedef int (*G65DDC_GET_GAIN)(int32_t hDevice,uint32_t DDC2ChannelId,double *Gain);
typedef int (*G65DDC_GET_CURRENT_GAIN)(int32_t hDevice,uint32_t DDC2ChannelId,double *Gain);
typedef int (*G65DDC_GET_SIGNAL_LEVEL)(int32_t hDevice,uint32_t DDC2ChannelId,float *Peak,float *RMS);
typedef int (*G65DDC_SET_DEMODULATOR_MODE)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t Mode);
typedef int (*G65DDC_GET_DEMODULATOR_MODE)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t *Mode);
typedef int (*G65DDC_SET_DEMODULATOR_FREQUENCY)(int32_t hDevice,uint32_t DDC2ChannelId,int32_t Frequency);
typedef int (*G65DDC_GET_DEMODULATOR_FREQUENCY)(int32_t hDevice,uint32_t DDC2ChannelId,int32_t *Frequency);
typedef int (*G65DDC_SET_DEMODULATOR_FILTER_BANDWIDTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t Bandwidth);
typedef int (*G65DDC_GET_DEMODULATOR_FILTER_BANDWIDTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t *Bandwidth);
typedef int (*G65DDC_SET_DEMODULATOR_FILTER_SHIFT)(int32_t hDevice,uint32_t DDC2ChannelId,int32_t Shift);
typedef int (*G65DDC_GET_DEMODULATOR_FILTER_SHIFT)(int32_t hDevice,uint32_t DDC2ChannelId,int32_t *Shift);
typedef int (*G65DDC_SET_DEMODULATOR_FILTER_LENGTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t Length);
typedef int (*G65DDC_GET_DEMODULATOR_FILTER_LENGTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t *Length);
typedef int (*G65DDC_SET_DEMODULATOR_PARAM)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t Code,const void *Buffer,uint32_t BufferSize);
typedef int (*G65DDC_GET_DEMODULATOR_PARAM)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t Code,void *Buffer,uint32_t BufferSize);
typedef int (*G65DDC_GET_DEMODULATOR_STATE)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t Code,void *Buffer,uint32_t BufferSize);
typedef int (*G65DDC_START_AUDIO)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t SampleSetsPerBuffer);
typedef int (*G65DDC_STOP_AUDIO)(int32_t hDevice,uint32_t DDC2ChannelId);
typedef int (*G65DDC_START_AUDIO_PLAYBACK)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t SampleSetsPerBuffer);
typedef int (*G65DDC_PAUSE_AUDIO_PLAYBACK)(int32_t hDevice,uint32_t DDC2ChannelId);
typedef int (*G65DDC_RESUME_AUDIO_PLAYBACK)(int32_t hDevice,uint32_t DDC2ChannelId);
typedef int (*G65DDC_SET_AUDIO_FILTER)(int32_t hDevice,uint32_t DDC2ChannelId,int Enabled);
typedef int (*G65DDC_GET_AUDIO_FILTER)(int32_t hDevice,uint32_t DDC2ChannelId,int *Enabled);
typedef int (*G65DDC_SET_AUDIO_FILTER_PARAMS)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t CutOffLow,uint32_t CutOffHigh,double Deemphasis);
typedef int (*G65DDC_GET_AUDIO_FILTER_PARAMS)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t *CutOffLow,uint32_t *CutOffHigh,double *Deemphasis);
typedef int (*G65DDC_SET_AUDIO_FILTER_LENGTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t Length);
typedef int (*G65DDC_GET_AUDIO_FILTER_LENGTH)(int32_t hDevice,uint32_t DDC2ChannelId,uint32_t *Length);
typedef int (*G65DDC_SET_AUDIO_GAIN)(int32_t hDevice,uint32_t DDC2ChannelId,double Gain);
typedef int (*G65DDC_GET_AUDIO_GAIN)(int32_t hDevice,uint32_t DDC2ChannelId,double *Gain);
typedef int (*G65DDC_SET_VOLUME)(int32_t hDevice,uint32_t DDC2ChannelId,uint8_t Volume);
typedef int (*G65DDC_GET_VOLUME)(int32_t hDevice,uint32_t DDC2ChannelId,uint8_t *Volume);
typedef int (*G65DDC_SET_MUTE)(int32_t hDevice,uint32_t DDC2ChannelId,int Mute);
typedef int (*G65DDC_GET_MUTE)(int32_t hDevice,uint32_t DDC2ChannelId,int *Mute);
typedef int (*G65DDC_SET_FREQUENCY)(int32_t hDevice,uint32_t DDC2ChannelId,int32_t Frequency);
typedef int (*G65DDC_GET_FREQUENCY)(int32_t hDevice,uint32_t DDC2ChannelId,int32_t *Frequency);
typedef int (*G65DDC_GET_SPECTRUM_COMPENSATION)(int32_t hDevice,int32_t Frequency,uint32_t Bandwidth,float *Buffer,uint32_t Count);
typedef int (*G65DDC_SET_NETWORK_PARAMS)(int32_t hDevice,const G65DDC_NETWORK_PARAMS *Params);
typedef int (*G65DDC_GET_NETWORK_PARAMS)(int32_t hDevice,G65DDC_NETWORK_PARAMS *Params);
typedef int (*G65DDC_GET_1PPS_COUNTERS)(int32_t hDevice,uint32_t *EventCounter,uint32_t *ADCPeriodCounter);
typedef int (*G65DDC_GET_1PPS_DDC1_COUNTERS)(int32_t hDevice,uint32_t DDC1ChannelId,double *SampleCounter,uint64_t *ADCPeriodCounter);
typedef int (*G65DDC_SET_EXTERNAL_ADC_CLOCK)(int32_t hDevice,int Enabled);
typedef int (*G65DDC_GET_EXTERNAL_ADC_CLOCK)(int32_t hDevice,int *Enabled);
typedef int (*G65DDC_SET_DAC)(int32_t hDevice,uint32_t DDC2ChannelId,int Enabled);
typedef int (*G65DDC_GET_DAC)(int32_t hDevice,uint32_t DDC2ChannelId,int *Enabled);

typedef void (*COH_G65DDC_ADC_SNAPSHOT_CALLBACK)(uint32_t DeviceIndex,const short *Buffer,uint32_t Count,uint32_t CenterFrequency,uint16_t ADCLevel,uintptr_t UserData);
typedef void (*COH_G65DDC_DDC1_STREAM_CALLBACK)(uint32_t DeviceCount,const void **Buffers,uint32_t Count,uint32_t BitsPerSample,uintptr_t UserData);
typedef void (*COH_G65DDC_DDC2_STREAM_CALLBACK)(uint32_t DeviceIndex,const float *Buffer,uint32_t Count,uintptr_t UserData);
typedef void (*COH_G65DDC_DDC2_PREPROCESSED_STREAM_CALLBACK)(uint32_t DeviceIndex,const float *Buffer,uint32_t Count,float SlevelPeak,float SlevelRMS,uintptr_t UserData);
typedef void (*COH_G65DDC_AUDIO_STREAM_CALLBACK)(uint32_t DeviceIndex,uint32_t Stage,const float *Buffer,uint32_t Count,uintptr_t UserData);

#pragma pack(push,1)

typedef struct
{
    COH_G65DDC_DDC1_STREAM_CALLBACK                 DDC1StreamCallback;
    COH_G65DDC_ADC_SNAPSHOT_CALLBACK                ADCSnapshotCallback;
    COH_G65DDC_DDC2_STREAM_CALLBACK                 DDC2StreamCallback;
    COH_G65DDC_DDC2_PREPROCESSED_STREAM_CALLBACK    DDC2PreprocessedStreamCallback;
    COH_G65DDC_AUDIO_STREAM_CALLBACK                AudioStreamCallback;
} COH_G65DDC_CALLBACKS;

typedef struct
{
    uint32_t            DeviceCount;
    G65DDC_DEVICE_INFO  *DeviceInfo;
} COH_G65DDC_DEVICE_SET;

typedef struct
{
    uint32_t                DeviceSetCount;
    COH_G65DDC_DEVICE_SET   *DeviceSet;
} COH_G65DDC_DEVICE_SET_LIST;

#pragma pack(pop)


typedef COH_G65DDC_DEVICE_SET_LIST* (*COH_G65DDC_GET_DEVICE_SET_LIST)(void);
typedef void (*COH_G65DDC_FREE_DEVICE_SET_LIST)(COH_G65DDC_DEVICE_SET_LIST *DeviceSetList);
typedef int32_t (*COH_G65DDC_OPEN_DEVICE_SET)(G65DDC_DEVICE_INFO *DeviceInfos,uint32_t Count);
typedef int (*COH_G65DDC_CLOSE_DEVICE_SET)(int32_t hDeviceSet);
typedef int (*COH_G65DDC_SET_CALLBACKS)(int32_t hDeviceSet,const COH_G65DDC_CALLBACKS *Callbacks,uintptr_t UserData);
typedef int (*COH_G65DDC_IS_CONNECTED)(int32_t hDeviceSet,int *Connected);
typedef int (*COH_G65DDC_GET_DEVICE_COUNT)(int32_t hDeviceSet,uint32_t *Count);
typedef int (*COH_G65DDC_GET_DEVICE_INFO)(int32_t hDeviceSet,uint32_t DeviceIndex,G65DDC_DEVICE_INFO *DeviceInfo);
typedef int (*COH_G65DDC_GET_DEVICE_INFOS)(int32_t hDeviceSet,G65DDC_DEVICE_INFO *Infos,uint32_t *InfoCount);
typedef int (*COH_G65DDC_SET_LED)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t LEDMode);
typedef int (*COH_G65DDC_GET_LED)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t *LEDMode);
typedef int (*COH_G65DDC_SET_POWER)(int32_t hDeviceSet,int Power);
typedef int (*COH_G65DDC_GET_POWER)(int32_t hDeviceSet,int *Power);
typedef int (*COH_G65DDC_GET_DEVICE_STATE)(int32_t hDeviceSet,uint32_t DeviceIndex,G65DDC_DEVICE_STATE *State);
typedef int (*COH_G65DDC_SET_BUILT_IN_TEST)(int32_t hDeviceSet,uint32_t DeviceIndex,int Enabled);
typedef int (*COH_G65DDC_GET_BUILT_IN_TEST)(int32_t hDeviceSet,uint32_t DeviceIndex,int *Enabled);
typedef int (*COH_G65DDC_SET_ATTENUATOR)(int32_t hDeviceSet,uint32_t Attenuator);
typedef int (*COH_G65DDC_GET_ATTENUATOR)(int32_t hDeviceSet,uint32_t *Attenuator);
typedef int (*COH_G65DDC_SET_PRESELECTORS)(int32_t hDeviceSet,uint32_t Low,uint32_t High);
typedef int (*COH_G65DDC_GET_PRESELECTORS)(int32_t hDeviceSet,uint32_t *Low,uint32_t *High);
typedef int (*COH_G65DDC_SET_PREAMPLIFIER)(int32_t hDeviceSet,int Preamp);
typedef int (*COH_G65DDC_GET_PREAMPLIFIER)(int32_t hDeviceSet,int *Preamp);
typedef int (*COH_G65DDC_SET_INVERTED)(int32_t hDeviceSet,int Inverted);
typedef int (*COH_G65DDC_GET_INVERTED)(int32_t hDeviceSet,int *Inverted);
typedef int (*COH_G65DDC_SET_RANGE)(int32_t hDeviceSet,uint32_t Range);
typedef int (*COH_G65DDC_GET_RANGE)(int32_t hDeviceSet,uint32_t *Range);
typedef int (*COH_G65DDC_SET_ADC_NOISE_BLANKER)(int32_t hDeviceSet,int Enabled);
typedef int (*COH_G65DDC_GET_ADC_NOISE_BLANKER)(int32_t hDeviceSet,int *Enabled);
typedef int (*COH_G65DDC_SET_ADC_NOISE_BLANKER_THRESHOLD)(int32_t hDeviceSet,uint16_t Threshold);
typedef int (*COH_G65DDC_GET_ADC_NOISE_BLANKER_THRESHOLD)(int32_t hDeviceSet,uint16_t *Threshold);
typedef int (*COH_G65DDC_START_ADC_SNAPSHOTS)(int32_t hDeviceSet,uint32_t DeviceIndex,uint16_t Interval,uint32_t SamplesPerSnapshot);
typedef int (*COH_G65DDC_STOP_ADC_SNAPSHOTS)(int32_t hDeviceSet,uint32_t DeviceIndex);
typedef int (*COH_G65DDC_GET_DDC_INFO)(int32_t hDeviceSet,uint32_t DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo);
typedef int (*COH_G65DDC_GET_DDC1_COUNT)(int32_t hDeviceSet,uint32_t *Count);
typedef int (*COH_G65DDC_SET_DDC1)(int32_t hDeviceSet,uint32_t DDCTypeIndex);
typedef int (*COH_G65DDC_GET_DDC1)(int32_t hDeviceSet,uint32_t *DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo);
typedef int (*COH_G65DDC_START_DDC1)(int32_t hDeviceSet,uint32_t SampleSetsPerBuffer);
typedef int (*COH_G65DDC_STOP_DDC1)(int32_t hDeviceSet);
typedef int (*COH_G65DDC_SET_DDC1_FREQUENCY)(int32_t hDeviceSet,int32_t Frequency);
typedef int (*COH_G65DDC_GET_DDC1_FREQUENCY)(int32_t hDeviceSet,int32_t *Frequency);
typedef int (*COH_G65DDC_SET_DDC1_PHASE_SHIFT)(int32_t hDeviceSet,uint32_t DeviceIndex,double PhaseShift);
typedef int (*COH_G65DDC_GET_DDC1_PHASE_SHIFT)(int32_t hDeviceSet,uint32_t DeviceIndex,double *PhaseShift);
typedef int (*COH_G65DDC_GET_DDC2_COUNT)(int32_t hDeviceSet,uint32_t *Count);
typedef int (*COH_G65DDC_SET_DDC2)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t DDCTypeIndex);
typedef int (*COH_G65DDC_GET_DDC2)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t *DDCTypeIndex,G65DDC_DDC_INFO *DDCInfo);
typedef int (*COH_G65DDC_START_DDC2)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t SampleSetsPerBuffer);
typedef int (*COH_G65DDC_STOP_DDC2)(int32_t hDeviceSet,uint32_t DeviceIndex);
typedef int (*COH_G65DDC_SET_DDC2_FREQUENCY)(int32_t hDeviceSet,uint32_t DeviceIndex,int32_t Frequency);
typedef int (*COH_G65DDC_GET_DDC2_FREQUENCY)(int32_t hDeviceSet,uint32_t DeviceIndex,int32_t *Frequency);
typedef int (*COH_G65DDC_SET_DDC2_NOISE_BLANKER)(int32_t hDeviceSet,uint32_t DeviceIndex,int Enabled);
typedef int (*COH_G65DDC_GET_DDC2_NOISE_BLANKER)(int32_t hDeviceSet,uint32_t DeviceIndex,int *Enabled);
typedef int (*COH_G65DDC_SET_DDC2_NOISE_BLANKER_THRESHOLD)(int32_t hDeviceSet,uint32_t DeviceIndex,double Threshold);
typedef int (*COH_G65DDC_GET_DDC2_NOISE_BLANKER_THRESHOLD)(int32_t hDeviceSet,uint32_t DeviceIndex,double *Threshold);
typedef int (*COH_G65DDC_GET_DDC2_NOISE_BLANKER_EXCESS_VALUE)(int32_t hDeviceSet,uint32_t DeviceIndex,double *Value);
typedef int (*COH_G65DDC_SET_NOTCH_FILTER)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,int Enabled);
typedef int (*COH_G65DDC_GET_NOTCH_FILTER)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,int *Enabled);
typedef int (*COH_G65DDC_SET_NOTCH_FILTER_FREQUENCY)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,int32_t Frequency);
typedef int (*COH_G65DDC_GET_NOTCH_FILTER_FREQUENCY)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,int32_t *Frequency);
typedef int (*COH_G65DDC_SET_NOTCH_FILTER_BANDWIDTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t Bandwidth);
typedef int (*COH_G65DDC_GET_NOTCH_FILTER_BANDWIDTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t *Bandwidth);
typedef int (*COH_G65DDC_SET_NOTCH_FILTER_LENGTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t Length);
typedef int (*COH_G65DDC_GET_NOTCH_FILTER_LENGTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t NotchFilterIndex,uint32_t *Length);
typedef int (*COH_G65DDC_SET_AGC)(int32_t hDeviceSet,uint32_t DeviceIndex,int Enabled);
typedef int (*COH_G65DDC_GET_AGC)(int32_t hDeviceSet,uint32_t DeviceIndex,int *Enabled);
typedef int (*COH_G65DDC_SET_AGC_PARAMS)(int32_t hDeviceSet,uint32_t DeviceIndex,double AttackTime,double DecayTime,double ReferenceLevel);
typedef int (*COH_G65DDC_GET_AGC_PARAMS)(int32_t hDeviceSet,uint32_t DeviceIndex,double *AttackTime,double *DecayTime,double *ReferenceLevel);
typedef int (*COH_G65DDC_SET_MAX_AGC_GAIN)(int32_t hDeviceSet,uint32_t DeviceIndex,double MaxGain);
typedef int (*COH_G65DDC_GET_MAX_AGC_GAIN)(int32_t hDeviceSet,uint32_t DeviceIndex,double *MaxGain);
typedef int (*COH_G65DDC_SET_GAIN)(int32_t hDeviceSet,uint32_t DeviceIndex,double Gain);
typedef int (*COH_G65DDC_GET_GAIN)(int32_t hDeviceSet,uint32_t DeviceIndex,double *Gain);
typedef int (*COH_G65DDC_GET_CURRENT_GAIN)(int32_t hDeviceSet,uint32_t DeviceIndex,double *Gain);
typedef int (*COH_G65DDC_GET_SIGNAL_LEVEL)(int32_t hDeviceSet,uint32_t DeviceIndex,float *Peak,float *RMS);
typedef int (*COH_G65DDC_SET_DEMODULATOR_MODE)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t Mode);
typedef int (*COH_G65DDC_GET_DEMODULATOR_MODE)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t *Mode);
typedef int (*COH_G65DDC_SET_DEMODULATOR_FREQUENCY)(int32_t hDeviceSet,uint32_t DeviceIndex,int32_t Frequency);
typedef int (*COH_G65DDC_GET_DEMODULATOR_FREQUENCY)(int32_t hDeviceSet,uint32_t DeviceIndex,int32_t *Frequency);
typedef int (*COH_G65DDC_SET_DEMODULATOR_FILTER_BANDWIDTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t Bandwidth);
typedef int (*COH_G65DDC_GET_DEMODULATOR_FILTER_BANDWIDTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t *Bandwidth);
typedef int (*COH_G65DDC_SET_DEMOUDLATOR_FILTER_SHIFT)(int32_t hDeviceSet,uint32_t DeviceIndex,int32_t Shift);
typedef int (*COH_G65DDC_GET_DEMOUDLATOR_FILTER_SHIFT)(int32_t hDeviceSet,uint32_t DeviceIndex,int32_t *Shift);
typedef int (*COH_G65DDC_SET_DEMOUDLATOR_FILTER_LENGTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t Length);
typedef int (*COH_G65DDC_GET_DEMOUDLATOR_FILTER_LENGTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t *Length);
typedef int (*COH_G65DDC_SET_DEMODULATOR_PARAM)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t Code,const void *Buffer,uint32_t BufferSize);
typedef int (*COH_G65DDC_GET_DEMODULATOR_PARAM)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t Code,void *Buffer,uint32_t BufferSize);
typedef int (*COH_G65DDC_GET_DEMODULATOR_STATE)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t Code,void *Buffer,uint32_t BufferSize);
typedef int (*COH_G65DDC_START_AUDIO)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t SampleSetsPerBuffer);
typedef int (*COH_G65DDC_STOP_AUDIO)(int32_t hDeviceSet,uint32_t DeviceIndex);
typedef int (*COH_G65DDC_SET_AUDIO_FILTER)(int32_t hDeviceSet,uint32_t DeviceIndex,int Enabled);
typedef int (*COH_G65DDC_GET_AUDIO_FILTER)(int32_t hDeviceSet,uint32_t DeviceIndex,int *Enabled);
typedef int (*COH_G65DDC_SET_AUDIO_FILTER_PARAMS)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t CutOffLow,uint32_t CutOffHigh,double Deemphasis);
typedef int (*COH_G65DDC_GET_AUDIO_FILTER_PARAMS)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t *CutOffLow,uint32_t *CutOffHigh,double *Deemphasis);
typedef int (*COH_G65DDC_SET_AUDIO_FILTER_LENGTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t Length);
typedef int (*COH_G65DDC_GET_AUDIO_FILTER_LENGTH)(int32_t hDeviceSet,uint32_t DeviceIndex,uint32_t *Length);
typedef int (*COH_G65DDC_SET_AUDIO_GAIN)(int32_t hDeviceSet,uint32_t DeviceIndex,double Gain);
typedef int (*COH_G65DDC_GET_AUDIO_GAIN)(int32_t hDeviceSet,uint32_t DeviceIndex,double *Gain);
typedef int (*COH_G65DDC_SET_VOLUME)(int32_t hDeviceSet,uint32_t DeviceIndex,uint8_t Volume);
typedef int (*COH_G65DDC_GET_VOLUME)(int32_t hDeviceSet,uint32_t DeviceIndex,uint8_t *Volume);
typedef int (*COH_G65DDC_SET_MUTE)(int32_t hDeviceSet,uint32_t DeviceIndex,int Mute);
typedef int (*COH_G65DDC_GET_MUTE)(int32_t hDeviceSet,uint32_t DeviceIndex,int *Mute);
typedef int (*COH_G65DDC_GET_SPECTRUM_COMPENSATION)(int32_t hDeviceSet,uint32_t DeviceIndex,int32_t Frequency,uint32_t Bandwidth,float *Buffer,uint32_t Count);
typedef int (*COH_G65DDC_SET_1PPS_ENABLED)(int32_t hDeviceSet,int Enabled);
typedef int (*COH_G65DDC_GET_1PPS_ENABLED)(int32_t hDeviceSet,int *Enabled);
typedef int (*COH_G65DDC_GET_1PPS_COUNTERS)(int32_t hDeviceSet,uint32_t *EventCounter,uint32_t *ADCPeriodCounter);
typedef int (*COH_G65DDC_GET_1PPS_DDC1_COUNTERS)(int32_t hDeviceSet,double *SampleCounter,uint64_t *ADCPeriodCounter);
typedef int (*COH_G65DDC_SET_TRIGGER_DELAY)(int32_t hDeviceSet,uint32_t Delay);
typedef int (*COH_G65DDC_GET_TRIGGER_DELAY)(int32_t hDeviceSet,uint32_t *Delay);
typedef int (*COH_G65DDC_SET_CONFIGURATION)(int32_t hDeviceSet,uint32_t Configuration);
typedef int (*COH_G65DDC_GET_CONFIGURATION)(int32_t hDeviceSet,uint32_t *Configuration);
typedef int (*COH_G65DDC_SET_EXTENDED_CONFIGURATION_PARAMS)(int32_t hDeviceSet,uint32_t Index,uint32_t DeviceCount);
typedef int (*COH_G65DDC_GET_EXTENDED_CONFIGURATION_PARAMS)(int32_t hDeviceSet,uint32_t *Index,uint32_t *DeviceCount);
typedef int (*COH_G65DDC_TRIGGER)(int32_t hDeviceSet);
typedef int (*COH_G65DDC_TRIGGER_TEST)(int32_t hDeviceSet);
typedef int (*COH_G65DDC_SET_DAC)(int32_t hDeviceSet,uint32_t DeviceIndex,int Enabled);
typedef int (*COH_G65DDC_GET_DAC)(int32_t hDeviceSet,uint32_t DeviceIndex,int *Enabled);


#endif // __G65DDCAPI_H__

